-- SSO DB DDL

CREATE TABLE schema_version (
    version NUMBER
);

INSERT INTO schema_version VALUES (5);

CREATE TABLE idp_info (
    id TEXT NOT NULL
);

CREATE TABLE sp (
    sp_id TEXT PRIMARY KEY, -- UUID
    name TEXT NOT NULL,
    cert TEXT NOT NULL,
    cert_old TEXT,
    api_url TEXT,
	name_mapping_url TEXT,
    url TEXT,
    api_version TEXT NOT NULL,
    idp_id TEXT,
    ui text default null,
    delete_time NUMERIC default NULL
);

CREATE TABLE sp_user (
    user_id TEXT PRIMARY KEY,
    sp_login TEXT NOT NULL,
    sp_id TEXT NOT NULL,
    delete_time NUMERIC default NULL,
    UNIQUE (sp_login, sp_id)
);

CREATE TABLE idp_session (
    session_id TEXT PRIMARY KEY,
    fed_id TEXT DEFAULT NULL,
    last_visit NUMERIC NOT NULL,
    expire_time NUMERIC NOT NULL
);

CREATE TABLE sp_session (
    sp_session_id TEXT PRIMARY KEY,
    session_id TEXT NOT NULL,
    user_id TEXT NOT NULL
);

CREATE TABLE federation (
    fed_id TEXT PRIMARY KEY,
    login TEXT UNIQUE,
    password TEXT,
    user_email TEXT,
    delete_time NUMERIC default NULL
);

CREATE TABLE federation_item (
    fed_item_id INTEGER PRIMARY KEY AUTOINCREMENT,
    fed_id TEXT NOT NULL,
    user_id TEXT NOT NULL,
    delete_time NUMERIC default NULL,
    UNIQUE (user_id)
);

--REPLACE INTO delegation (d_id, src_user, dst_user, delete_time) VALUES (?,?,?,?)

CREATE TABLE delegation (
    d_id INTEGER PRIMARY KEY AUTOINCREMENT,
    src_user TEXT NOT NULL,
    dst_user TEXT NOT NULL,
    delete_time NUMERIC default NULL,
    UNIQUE (src_user, dst_user) ON CONFLICT IGNORE
);

CREATE TABLE delegation_pair (
    src_user TEXT NOT NULL,
    dst_user TEXT NOT NULL,
    level NUMERIC NOT NULL,
    PRIMARY KEY (src_user, dst_user)
);

CREATE INDEX psindex ON delegation_pair(src_user);
CREATE INDEX pdindex ON delegation_pair(dst_user);

-- on delete from SP cascade
CREATE TRIGGER fkd_sp_user_sp_id
BEFORE DELETE ON sp
FOR EACH ROW BEGIN
  DELETE FROM sp_user WHERE sp_id = OLD.sp_id;
END;

-- on update delete_time from SP cascade
CREATE TRIGGER fkud_sp_user_sp_id
after update OF delete_time ON sp
FOR EACH ROW BEGIN
  UPDATE sp_user SET delete_time = new.delete_time WHERE sp_id = NEW.sp_id;
END;

-- on delete from SP USER cascade
CREATE TRIGGER fkd_sp_user_user_id
BEFORE DELETE ON sp_user
FOR EACH ROW BEGIN
  DELETE FROM federation_item WHERE user_id = OLD.user_id;
  DELETE FROM sp_session WHERE user_id = OLD.user_id;
END;

-- on update of delete_time from SP USER cascade
CREATE TRIGGER fku_sp_user_user_id
AFTER UPDATE OF delete_time ON sp_user
FOR EACH ROW BEGIN
  UPDATE federation_item SET delete_time = NEW.delete_time WHERE user_id = NEW.user_id;
  DELETE FROM sp_session WHERE user_id = NEW.user_id;
END;

-- on delete from Federation cascade
CREATE TRIGGER fkd_fed_id
BEFORE DELETE ON federation
FOR EACH ROW BEGIN
  DELETE FROM federation_item WHERE fed_id = OLD.fed_id;
  DELETE FROM idp_session WHERE fed_id = OLD.fed_id;
END;

-- on delete_time update of Federation cascade
CREATE TRIGGER fku_fed_id
AFTER UPDATE OF delete_time ON federation
FOR EACH ROW BEGIN
  UPDATE federation_item SET delete_time = NEW.delete_time WHERE fed_id = NEW.fed_id;
  DELETE FROM idp_session WHERE fed_id = NEW.fed_id;
END;

-- on delete from IdP Session cascade
CREATE TRIGGER fkd_session_id
BEFORE DELETE ON idp_session
FOR EACH ROW BEGIN
  DELETE FROM sp_session WHERE session_id = OLD.session_id;
END;

-- on insert into delegations
CREATE TRIGGER tr_deleg_insert after INSERT ON delegation
begin
  SELECT CASE
    WHEN ((
      SELECT COUNT(user_id)
      FROM sp_user
      WHERE user_id = new.src_user OR user_id = new.dst_user) < 2)
    THEN RAISE(ABORT, 'One of users does not exist. Foreign key violation')
    WHEN ((
      SELECT COUNT(*) FROM delegation_pair
      WHERE src_user = new.dst_user AND dst_user = new.src_user) > 0)
    THEN RAISE(ABORT, 'No cycles in delegations graph allowed')
  END;
  INSERT OR REPLACE INTO delegation_pair VALUES (new.src_user, new.dst_user, 1);
  INSERT OR IGNORE INTO delegation_pair
      SELECT x1.src_user, new.dst_user, x1.level + 1
      FROM delegation_pair x1
      WHERE x1.dst_user = new.src_user
    UNION
      SELECT new.src_user, x2.dst_user, x2.level + 1
      FROM delegation_pair x2
      WHERE x2.src_user = new.dst_user
    UNION
      SELECT x3.src_user, x4.dst_user, x3.level + x4.level + 1
      FROM delegation_pair x3, delegation_pair x4
      WHERE x3.dst_user = new.src_user AND x4.src_user = new.dst_user;
end;

CREATE TRIGGER tr_deleg_upd before UPDATE OF src_user, dst_user ON delegation
begin
  select RAISE(ABORT, 'Delegations update not allowed');
end;

-- http://citeseer.ist.psu.edu/dong99maintaining.html
CREATE TABLE suspect (src_user text, dst_user text);
CREATE TABLE trusty (src_user text, dst_user text);
CREATE TABLE delegation_pair_new (src_user text, dst_user text);
CREATE TRIGGER tr_deleg_del before UPDATE OF delete_time ON delegation
begin
  SELECT CASE
    WHEN ((NEW.delete_time is null))
    THEN RAISE(ABORT, 'Deleted delegations restore available with external utility only')
  end;

  delete from trusty;
  delete from suspect;
  delete from delegation_pair_new;

  INSERT INTO suspect
    SELECT X.src_user, Y.dst_user
    FROM delegation_pair AS X, delegation_pair AS Y
    WHERE X.dst_user = old.src_user AND Y.src_user = old.dst_user
  UNION
    SELECT X.src_user, old.dst_user
    FROM delegation_pair AS X WHERE X.dst_user = old.src_user
  UNION
    SELECT old.src_user, X.dst_user
    FROM delegation_pair AS X WHERE X.src_user = old.dst_user
  UNION
    SELECT old.src_user, old.dst_user
    FROM delegation_pair AS X WHERE X.src_user = old.src_user and X.dst_user = old.dst_user;

  INSERT INTO TRUSTY
  SELECT * FROM (
    SELECT src_user, dst_user FROM delegation_pair d WHERE NOT EXISTS (
      SELECT * FROM suspect
      WHERE suspect.src_user = d.src_user
      AND suspect.dst_user = d.dst_user
    )
    UNION
      SELECT src_user, dst_user FROM delegation g
      WHERE g.src_user != old.src_user AND g.dst_user != old.dst_user
  );

  INSERT INTO delegation_pair_new
    SELECT * FROM TRUSTY
    UNION
      SELECT T1.src_user, T2.dst_user FROM TRUSTY T1, TRUSTY T2
      WHERE T1.dst_user = T2.src_user
    UNION
      SELECT T1.src_user, T3.dst_user FROM TRUSTY T1, TRUSTY T2, TRUSTY T3
      WHERE T1.dst_user = T2.src_user AND T2.dst_user = T3.src_user;

  DELETE FROM delegation_pair WHERE NOT EXISTS (
    SELECT * FROM delegation_pair_new T
    WHERE T.src_user = delegation_pair.src_user
      AND T.dst_user = delegation_pair.dst_user
  );
end;


-- sp_user.sp_id foreign key
CREATE TRIGGER fki_sp_user_sp_id
BEFORE INSERT ON sp_user
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT sp_id FROM sp WHERE sp_id = NEW.sp_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "sp_user" voilates foreign key sp_user.sp_id')
  END;
  DELETE from sp_user where sp_id = new.sp_id and sp_login = new.sp_login and delete_time is not null;
END;

CREATE TRIGGER fku_sp_user_sp_id
BEFORE UPDATE ON sp_user
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT sp_id FROM sp WHERE sp_id = NEW.sp_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "sp_user" violates foreign key sp_user.sp_id')
  END;
  DELETE from sp_user where sp_id = old.sp_id and sp_login = old.sp_login and delete_time is not null;
END;

-- fed_item.user_id foreign key
CREATE TRIGGER fki_fed_item_user_id
BEFORE INSERT ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "federation_item" voilates foreign key user_id')
  END;
END;

CREATE TRIGGER fku_fed_item_user_id
BEFORE UPDATE ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "federation_item" voilates foreign key user_id')
  END;
END;

-- fed_item.fed_id foreign key
CREATE TRIGGER fki_fed_item_fed_id
BEFORE INSERT ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "federation_item" voilates foreign key fed_id')
  END;
END;

CREATE TRIGGER fku_fed_item_fed_id
BEFORE UPDATE ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "federation_item" voilates foreign key fed_id')
  END;
END;

-- idp_session.fed_id foreign key
CREATE TRIGGER fki_idp_session_fed_id
BEFORE INSERT ON idp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((NEW.fed_id IS NOT NULL) AND (SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "idp_session" voilates foreign key fed_id')
  END;
END;

CREATE TRIGGER fku_idp_session_fed_id
BEFORE UPDATE ON idp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((NEW.fed_id IS NOT NULL) AND (SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "idp_session" voilates foreign key fed_id')
  END;
END;

-- sp_session.session_id foreign key
CREATE TRIGGER fki_sp_session_session_id
BEFORE INSERT ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT session_id FROM idp_session WHERE session_id = NEW.session_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "sp_session" voilates foreign key session_id')
  END;
END;

CREATE TRIGGER fku_sp_session_session_id
BEFORE UPDATE ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT session_id FROM idp_session WHERE session_id = NEW.session_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "sp_session" voilates foreign key session_id')
  END;
END;

-- sp_session.sp_id foreign key
CREATE TRIGGER fki_sp_session_user_id
BEFORE INSERT ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "sp_session" voilates foreign key user_id')
  END;
END;

CREATE TRIGGER fku_sp_session_user_id
BEFORE UPDATE ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "sp_session" voilates foreign key user_id')
  END;
END;

CREATE TRIGGER fki_idp_info
BEFORE INSERT ON idp_info
BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM idp_info) IS NOT NULL)
    THEN RAISE(ABORT, 'idp_info is read-only')
  END;
END;

CREATE TRIGGER fku_idp_info
BEFORE UPDATE ON idp_info
BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM idp_info) IS NOT NULL)
    THEN RAISE(ABORT, 'idp_info is read-only')
  END;
END;