-- SSO DB DDL

CREATE TABLE schema_version (
    version NUMBER
);

INSERT INTO schema_version VALUES (9);

CREATE TABLE idp_info (
    id TEXT NOT NULL
);

CREATE TABLE sp (
    sp_id TEXT PRIMARY KEY, -- UUID
    name TEXT NOT NULL,
    cert TEXT NOT NULL,
    cert_old TEXT,
    api_url TEXT,
    name_mapping_url TEXT,
    url TEXT,
    api_version TEXT NOT NULL,
    idp_id TEXT,
    ui text default null,
    delete_time NUMERIC default NULL
);

CREATE TABLE sp_user (
    user_id TEXT PRIMARY KEY,
    sp_login TEXT NOT NULL,
    sp_id TEXT NOT NULL,
    delete_time NUMERIC default NULL,
    UNIQUE (sp_login, sp_id)
);

CREATE TABLE idp_session (
    session_id TEXT PRIMARY KEY,
    fed_id TEXT DEFAULT NULL,
    last_visit NUMERIC NOT NULL,
    expire_time NUMERIC NOT NULL
);

CREATE TABLE sp_session (
    sp_session_id TEXT PRIMARY KEY,
    session_id TEXT NOT NULL,
    user_id TEXT NOT NULL,
    referer_scheme TEXT NULL,
    referer_host TEXT NULL,
    referer_port TEXT NULL
);

CREATE TABLE federation (
    fed_id TEXT PRIMARY KEY,
    master_sp_id TEXT DEFAULT NULL,
    login TEXT UNIQUE,
    password TEXT,
    user_email TEXT,
    delete_time NUMERIC default NULL
);

CREATE TABLE federation_item (
    fed_item_id INTEGER PRIMARY KEY,
    fed_id TEXT NOT NULL,
    user_id TEXT NOT NULL,
    delete_time NUMERIC default NULL,
    UNIQUE (user_id)
);

create table selector_types (
    id text primary key,
    expression text not null
);

insert into selector_types (id, expression) values ('u', 'sp_user.user_id');
insert into selector_types (id, expression) values ('sp', 'sp.sp_id');

create table delegation (
    id integer primary key,
    src_type integer not null,
    src_value text not null,
    dst_type integer not null,
    dst_value text not null,
    delete_time numeric default null,
    unique (src_type, src_value, dst_type, dst_value) on conflict ignore
);
create index idx_delegation_src_selector on delegation(src_type, src_value);
create index idx_delegation_dst_selector on delegation(dst_type, dst_value);

-- on delete from SP cascade
CREATE TRIGGER fkd_sp_user_sp_id
BEFORE DELETE ON sp
FOR EACH ROW BEGIN
  DELETE FROM sp_user WHERE sp_id = OLD.sp_id;
END;

-- on update delete_time from SP cascade
CREATE TRIGGER fkud_sp_user_sp_id
after update OF delete_time ON sp
FOR EACH ROW BEGIN
  UPDATE sp_user SET delete_time = new.delete_time WHERE sp_id = NEW.sp_id;
END;

-- on delete from SP USER cascade
CREATE TRIGGER fkd_sp_user_user_id
BEFORE DELETE ON sp_user
FOR EACH ROW BEGIN
  DELETE FROM federation_item WHERE user_id = OLD.user_id;
  DELETE FROM sp_session WHERE user_id = OLD.user_id;
END;

-- on update of delete_time from SP USER cascade
CREATE TRIGGER fku_sp_user_user_id
AFTER UPDATE OF delete_time ON sp_user
FOR EACH ROW BEGIN
  UPDATE federation_item SET delete_time = NEW.delete_time WHERE user_id = NEW.user_id;
  DELETE FROM sp_session WHERE user_id = NEW.user_id;
END;

-- on delete from Federation cascade
CREATE TRIGGER fkd_fed_id
BEFORE DELETE ON federation
FOR EACH ROW BEGIN
  DELETE FROM federation_item WHERE fed_id = OLD.fed_id;
  UPDATE idp_session SET fed_id = NULL WHERE fed_id = OLD.fed_id;
END;

-- on delete_time update of Federation cascade
CREATE TRIGGER fku_fed_id
AFTER UPDATE OF delete_time ON federation
FOR EACH ROW BEGIN
  UPDATE federation_item SET delete_time = NEW.delete_time WHERE fed_id = NEW.fed_id;
  UPDATE idp_session SET fed_id = NULL WHERE fed_id = NEW.fed_id;
END;

-- on delete from IdP Session cascade
CREATE TRIGGER fkd_session_id
BEFORE DELETE ON idp_session
FOR EACH ROW BEGIN
  DELETE FROM sp_session WHERE session_id = OLD.session_id;
END;

-- sp_user.sp_id foreign key
CREATE TRIGGER fki_sp_user_sp_id
BEFORE INSERT ON sp_user
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT sp_id FROM sp WHERE sp_id = NEW.sp_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "sp_user" voilates foreign key sp_user.sp_id')
  END;
  DELETE from sp_user where sp_id = new.sp_id and sp_login = new.sp_login and delete_time is not null;
END;

CREATE TRIGGER fku_sp_user_sp_id
BEFORE UPDATE ON sp_user
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT sp_id FROM sp WHERE sp_id = NEW.sp_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "sp_user" violates foreign key sp_user.sp_id')
  END;
  DELETE from sp_user where sp_id = old.sp_id and sp_login = old.sp_login and delete_time is not null;
END;

-- fed_item.user_id foreign key
CREATE TRIGGER fki_fed_item_user_id
BEFORE INSERT ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "federation_item" voilates foreign key user_id')
  END;
END;

CREATE TRIGGER fku_fed_item_user_id
BEFORE UPDATE ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "federation_item" voilates foreign key user_id')
  END;
END;

-- fed_item.fed_id foreign key
CREATE TRIGGER fki_fed_item_fed_id
BEFORE INSERT ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "federation_item" voilates foreign key fed_id')
  END;
END;

CREATE TRIGGER fku_fed_item_fed_id
BEFORE UPDATE ON federation_item
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "federation_item" voilates foreign key fed_id')
  END;
END;

-- idp_session.fed_id foreign key
CREATE TRIGGER fki_idp_session_fed_id
BEFORE INSERT ON idp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((NEW.fed_id IS NOT NULL) AND (SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "idp_session" voilates foreign key fed_id')
  END;
END;

CREATE TRIGGER fku_idp_session_fed_id
BEFORE UPDATE ON idp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((NEW.fed_id IS NOT NULL) AND (SELECT fed_id FROM federation WHERE fed_id = NEW.fed_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "idp_session" voilates foreign key fed_id')
  END;
END;

-- sp_session.session_id foreign key
CREATE TRIGGER fki_sp_session_session_id
BEFORE INSERT ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT session_id FROM idp_session WHERE session_id = NEW.session_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "sp_session" voilates foreign key session_id')
  END;
END;

CREATE TRIGGER fku_sp_session_session_id
BEFORE UPDATE ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT session_id FROM idp_session WHERE session_id = NEW.session_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "sp_session" voilates foreign key session_id')
  END;
END;

-- sp_session.sp_id foreign key
CREATE TRIGGER fki_sp_session_user_id
BEFORE INSERT ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "sp_session" voilates foreign key user_id')
  END;
END;

CREATE TRIGGER fku_sp_session_user_id
BEFORE UPDATE ON sp_session
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT user_id FROM sp_user WHERE user_id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "sp_session" voilates foreign key user_id')
  END;
END;

CREATE TRIGGER fki_idp_info
BEFORE INSERT ON idp_info
BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM idp_info) IS NOT NULL)
    THEN RAISE(ABORT, 'idp_info is read-only')
  END;
END;

CREATE TRIGGER fku_idp_info
BEFORE UPDATE ON idp_info
BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM idp_info) IS NOT NULL)
    THEN RAISE(ABORT, 'idp_info is read-only')
  END;
END;